@@ -14,16 +14,18 @@ from typing import Union, Optional, List
 import pytz
 import twisted
 from twisted.internet import reactor
-from twisted.internet import task, defer
-from twisted.internet.defer import Deferred
+from twisted.internet import task
+from twisted.internet.defer import Deferred, inlineCallbacks
 from twisted.internet.error import ConnectionDone, ConnectionRefusedError
 from twisted.internet.protocol import connectionDone, ReconnectingClientFactory
 
-from vortex.DeferUtil import vortexLogFailure
+from vortex.DeferUtil import vortexLogFailure, isMainThread
 from vortex.PayloadEnvelope import PayloadEnvelope, VortexMsgList
+from vortex.PayloadPriority import DEFAULT_PRIORITY
 from vortex.VortexABC import VortexABC, VortexInfo
 from vortex.VortexPayloadProtocol import VortexPayloadProtocol
 from vortex.VortexServer import HEART_BEAT_PERIOD
+from vortex.VortexWritePushProducer import VortexWritePushProducer
 
 logger = logging.getLogger(name=__name__)
 
@@ -40,6 +42,8 @@ class VortexPayloadTcpClientProtocol(VortexPayloadProtocol):
         d = self._sendBeatLoopingCall.start(HEART_BEAT_PERIOD, now=False)
         d.addErrback(lambda f: logger.exception(f.value))
 
+        self._producer: Optional[VortexWritePushProducer] = None
+
     def _beat(self):
         if self._vortexClient:
             self._vortexClient._beat()
@@ -48,13 +52,17 @@ class VortexPayloadTcpClientProtocol(VortexPayloadProtocol):
         from vortex.VortexFactory import VortexFactory
         VortexFactory._notifyOfVortexStatusChange(name, online=True)
 
+        self._producer.setRemoteVortexName(self._serverVortexName)
+
         if self._vortexClient:
             self._vortexClient._setNameAndUuid(name=self._serverVortexName,
                                                uuid=self._serverVortexUuid)
 
     def _createResponseSenderCallable(self):
-        def sendResponse(vortexMsgs: Union[VortexMsgList, bytes]):
-            return self._vortexClient.sendVortexMsg(vortexMsgs=vortexMsgs)
+        def sendResponse(vortexMsgs: Union[VortexMsgList, bytes],
+                         priority: int = DEFAULT_PRIORITY):
+            return self._vortexClient.sendVortexMsg(vortexMsgs=vortexMsgs,
+                                                    priority=priority)
 
         return sendResponse
 
@@ -63,25 +71,31 @@ class VortexPayloadTcpClientProtocol(VortexPayloadProtocol):
             return
 
         # Send the heartbeats
-        self.transport.write(b'.')
+        self._producer.write(b'.', DEFAULT_PRIORITY)
 
-    def write(self, payloadVortexStr: bytes):
+    def write(self, payloadVortexStr: bytes, priority: int = DEFAULT_PRIORITY):
         if not twisted.python.threadable.isInIOThread():
             e = Exception("Write called from NON main thread")
             logger.exception(str(e))
             raise e
 
         assert not self._closed
-        self.transport.write(payloadVortexStr)
-        self.transport.write(b'.')
+        self._producer.write(payloadVortexStr + b'.', priority)
 
     def connectionMade(self):
+        self._producer = VortexWritePushProducer(self.transport, lambda: self.close())
+
+        # Register the producer if there isn't one already.
+        if not self.transport.producer:
+            self.transport.registerProducer(self._producer, True)
+
         # Send a heart beat down the new connection, tell it who we are.
         connectPayloadFilt = {
             PayloadEnvelope.vortexUuidKey: self._vortexClient.uuid,
             PayloadEnvelope.vortexNameKey: self._vortexClient.name
         }
-        self.write(PayloadEnvelope(filt=connectPayloadFilt).toVortexMsg())
+        self._producer.write(PayloadEnvelope(filt=connectPayloadFilt).toVortexMsg(),
+                             DEFAULT_PRIORITY)
 
     def connectionLost(self, reason=connectionDone):
         from vortex.VortexFactory import VortexFactory
@@ -224,7 +238,13 @@ class VortexClientTcp(ReconnectingClientFactory, VortexABC):
 
     def sendVortexMsg(self,
                       vortexMsgs: Union[VortexMsgList, bytes, None] = None,
-                      vortexUuid: Optional[str] = None) -> Deferred:
+                      vortexUuid: Optional[str] = None,
+                      priority: int = DEFAULT_PRIORITY) -> Deferred:
+        """ Send Vortex Msg
+
+        NOTE: Priority ins't supported as there is no buffer for this class.
+
+        """
 
         if vortexMsgs is None:
             vortexMsgs = self._reconnectVortexMsgs
@@ -239,9 +259,14 @@ class VortexClientTcp(ReconnectingClientFactory, VortexABC):
         # if not self.__protocol.serverVortexUuid:
         #     return []
 
+        if isMainThread():
+            return self._sendVortexMsgLater(vortexMsgs)
+
         return task.deferLater(reactor, 0, self._sendVortexMsgLater, vortexMsgs)
 
+    @inlineCallbacks
     def _sendVortexMsgLater(self, vortexMsgs: VortexMsgList):
+        yield None
         assert self._server
         assert vortexMsgs
 
@@ -250,7 +275,7 @@ class VortexClientTcp(ReconnectingClientFactory, VortexABC):
         for vortexMsg in vortexMsgs:
             self.__protocol.write(vortexMsg)
 
-        return defer.succeed(True)
+        return True
 
     def _beat(self):
         """ Beat, Called by protocol """
@@ -282,7 +307,6 @@ class VortexClientTcp(ReconnectingClientFactory, VortexABC):
             self._reconnectAfterHeartBeatLost()
             return
 
-
     def _reconnectAfterHeartBeatLost(self):
         if self._retrying:
             return
@@ -293,4 +317,4 @@ class VortexClientTcp(ReconnectingClientFactory, VortexABC):
                     self._server, self._port)
 
         if self.__protocol:
-            self.__protocol.close()
\ No newline at end of file
+            self.__protocol.close()
